package {
	import com.adobe.utils.AGALMiniAssembler;
	import com.adobe.utils.PerspectiveMatrix3D;
	
	import flash.display.Sprite;
	import flash.display.Stage3D;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.display3D.Context3D;
	import flash.display3D.Context3DProgramType;
	import flash.display3D.Context3DRenderMode;
	import flash.display3D.Context3DTriangleFace;
	import flash.display3D.Context3DVertexBufferFormat;
	import flash.display3D.IndexBuffer3D;
	import flash.display3D.Program3D;
	import flash.display3D.VertexBuffer3D;
	import flash.events.ErrorEvent;
	import flash.events.Event;
	import flash.geom.Matrix3D;
	import flash.geom.Vector3D;
	
	public class Context3DExample extends Sprite {
		public const zNear:Number = 1;
		public const zFar:Number = 500;
		
		public const fov:Number = 45;
		
		private var stage3D:Stage3D;
		private var renderContext:Context3D;
		private var vBox:Vector.<Box3D>;
		
		private var projection:PerspectiveMatrix3D = new PerspectiveMatrix3D();
		private var world:Matrix3D = new Matrix3D();
		private var view:Matrix3D = new Matrix3D();
		private var finalTransform:Matrix3D = new Matrix3D();
		
		//For rotating the cube
		private const pivot:Vector3D = new Vector3D();
		
		private const VERTEX_SHADER:String = "m44 op, va0, vc0    \n" + // 4x4 matrix transform 
			"mov v0, va1"; //copy color to varying variable v0
		
		private const FRAGMENT_SHADER:String = "mov oc, v0"; //Set the output color to the value interpolated from the three triangle vertices 
		
		private var vertexAssembly:AGALMiniAssembler = new AGALMiniAssembler();
		private var fragmentAssembly:AGALMiniAssembler = new AGALMiniAssembler();
		private var programPair:Program3D;
		
		public function Context3DExample(){
			this.stage.scaleMode = StageScaleMode.NO_SCALE;
			this.stage.align = StageAlign.TOP_LEFT;
			
			stage3D = this.stage.stage3Ds[0];
			stage3D.x = 0;
			stage3D.y = 0;
			
			//Add event listener before requesting the context
			stage3D.addEventListener(Event.CONTEXT3D_CREATE, contextCreated);
			stage3D.addEventListener(ErrorEvent.ERROR, contextCreationError);
			stage3D.requestContext3D(Context3DRenderMode.AUTO);
			
			//Compile shaders
			vertexAssembly.assemble(Context3DProgramType.VERTEX, VERTEX_SHADER, false);
			fragmentAssembly.assemble(Context3DProgramType.FRAGMENT, FRAGMENT_SHADER, false);
		}
		
		//Note, context3DCreate event can happen at any time, such as when the hardware resources are taken by another process
		private function contextCreated(event:Event):void {
			renderContext = Stage3D(event.target).context3D;
			setupScene();
		}
		
		private function setupScene():void {
			renderContext.enableErrorChecking = true; //Can slow rendering - only turn on when developing/testing
			renderContext.configureBackBuffer(stage.stageWidth, stage.stageHeight, 2, true);
			renderContext.setCulling(Context3DTriangleFace.BACK);
			
			vBox = new Vector.<Box3D>();
			var box:Box3D;
			for(var i:int = 0; i < 10; i++) {
				box = new Box3D(renderContext);
				box.getMatrix3D().appendTranslation(-5+Math.random() * 10, -5+Math.random() * 10, -5+Math.random() * 10);
				vBox.push(box);
			}
			
			//Upload programs to render context
			programPair = renderContext.createProgram();
			programPair.upload(vertexAssembly.agalcode, fragmentAssembly.agalcode);
			renderContext.setProgram(programPair);
			
			//Set up 3D transforms
			projection.perspectiveFieldOfViewRH(fov, stage.stageWidth / stage.stageHeight, zNear, zFar);
			view.appendTranslation(0, 0, -10);
			this.stage.addEventListener(Event.ENTER_FRAME, render);
		}
		
		private function render(event:Event):void {
			//Clear is required before drawTriangles on each frame
			renderContext.clear(.3, .3, .3);
			
			//Draw the 12 triangles that make up the cube
			for each(var box:Box3D in vBox) {
				//Combine transforms
				finalTransform.identity();
				finalTransform.append(box.getMatrix3D());
				finalTransform.append(world);
				finalTransform.append(view);
				finalTransform.append(projection);
				
				//Pass the final transform to the vertex shader as program constant, vc0
				renderContext.setProgramConstantsFromMatrix(Context3DProgramType.VERTEX, 0, finalTransform, true);
				
				box.render(renderContext);
			}
			
			//Show the frame
			renderContext.present();
		}
		
		private function contextCreationError(error:ErrorEvent):void {
			trace(error.errorID + ": " + error.text);
		}
	}
}